import java.util.Iterator;
import java.util.NoSuchElementException;

/** rewritten Booklist example with interface 
 *  @author abx
 */

public class BookListWithArray implements BookList, Iterator<Book>, Iterable<Book> {

	private Book[] list;
	private int current; //counter used in the iterator implementation 
	
	public BookListWithArray() {
		list = new Book[0];
		current = 0;
	}
	
    /** Add a book to the beginning of the array
	@param newBook the Book to be added */
    public boolean add(Book newBook) {
		Book[] temp = new Book[list.length+1];
		int i;
		temp[0] = newBook;
		for (i = 0; i < list.length; i++) {
			temp[i+1] = list[i];
		}
		list = temp;
		return true;
    }

	public void addFirst(Book newBook) {
		// move each element up one
		add(get(size()-1));
		for (int i=0; i<size()-1; i++) {
			list[i+1] = list[i];
		}
		list[0] = newBook;
	}

	public Book get(int position) {
    	return list[position];
	}

    /** Insert a book 
	@param newBook the Book to be inserted
	@position the index of the list nodes 
	before the insertion point (counting from 0)*/
    public void insert(Book newBook, int position) {
		if (position < 0 || position > list.length-1) {
			System.out.print("ERROR: Invalid value for " + "insertion index: ");
			System.out.println(position);
			System.out.println("(Insertion skipped)");
			return;
		}

		Book[] temp = new Book[list.length+1];
		int i;
		for (i = 0; i <= position; i++) { 
	    	temp[i] = list[i];
		}
		temp[position+1] = newBook;
		for (i = position+2; i <= list.length; i++) {
			temp[i] = list[i-1];
		}
		list = temp;	
    }

    /** @return length of list */
    public int size() {
		return list.length;
    }

	/** @return true if list has no elements */
    public boolean isEmpty() {
		return (size() < 1);
    }

    /** @return a string which represents the list data */
    public String toString() {
		String result = "Array implemented Booklist:\n";
		/* for (int i=0; i<list.length; i++) {
			result += list[i].toString() + "\n";
		} */
		current = 0;
		while (hasNext()) {
			result += next().toString() + "\n";
		}
		return result;
	}
	/** the Iterator methods, no further comment */
	public boolean hasNext() {
		return current < list.length;
	}
	
	public Book next() {
		try {
			return list[current++];
		} catch (NoSuchElementException e) {
			System.out.println(e);
			return null;
		}
	}

	public void remove() throws UnsupportedOperationException { }
	
	/** the Iterable iterator() method returns the object itself */
	public Iterator<Book> iterator() {
		current = 0;
		return this;
	}
}
	